@@ -83,8 +83,8 @@ void init_clipboard(void) {
 	HANDLE ht;
 	DWORD tid;
 
-	ht = CreateThread(NULL,0,(LPTHREAD_START_ROUTINE)clipper_thread,
-					NULL,0,&tid);
+	ht = CreateThread(NULL,gdwStackSize,
+					(LPTHREAD_START_ROUTINE)clipper_thread, NULL,0,&tid);
 
 	if (!ht)
 		abort();
@@ -364,7 +364,7 @@ HANDLE create_clip_writer_thread(void) {
 		abort();
 	}
 	is_dev_clipboard_active = 1;
-	ghdevclipthread = CreateThread(NULL,0,
+	ghdevclipthread = CreateThread(NULL,gdwStackSize,
 				(LPTHREAD_START_ROUTINE)clip_writer_proc, hread,0,&tid);
 //	CloseHandle(ht);
 	return hwrite;
@@ -438,8 +438,100 @@ HANDLE create_clip_reader_thread(void) {
 		abort();
 	}
 	is_dev_clipboard_active = 1;
-	ghdevclipthread = CreateThread(NULL,0,
+	ghdevclipthread = CreateThread(NULL,gdwStackSize,
 				(LPTHREAD_START_ROUTINE)clip_reader_proc, hwrite,0,&tid);
-//	CloseHandle(ht);
 	return hread;
 }
+
+/* These are here since we already include ed.h here */
+extern void	 c_delafter(int);
+extern void	 c_delbefore(int);
+CCRETVAL
+e_dosify_next(c)
+    int c;
+{
+    register Char *cp, *p, *kp;
+
+    USE(c);
+    if (Cursor == LastChar)
+	return(CC_ERROR);
+    /* else */
+
+	cp = Cursor;
+	while(  cp < LastChar) {
+		if ( (*cp & CHAR == ' ') && (cp[-1] & CHAR != '\\') )
+			break;
+		cp++;
+	}
+
+    for (p = Cursor, kp = KillBuf; p < cp; p++)	{/* save the text */
+	if ( ( *p & CHAR ) == '/') {
+	    *kp++ = '\\';
+	    *kp++ = '\\';
+	}
+	else
+	    *kp++ = *p;
+    }
+    LastKill = kp;
+
+    c_delafter((int)(cp - Cursor));	/* delete after dot */
+    if (Cursor > LastChar)
+	Cursor = LastChar;	/* bounds check */
+    return (e_yank_kill(c));
+}
+/*ARGSUSED*/
+CCRETVAL
+e_dosify_prev(c)
+    int c;
+{
+    register Char *cp, *p, *kp;
+
+    USE(c);
+    if (Cursor == InputBuf)
+	return(CC_ERROR);
+    /* else */
+
+    cp = Cursor-1;
+    /* Skip trailing spaces */
+    while ((cp > InputBuf) && ( (*cp & CHAR) == ' '))
+    	cp--;
+
+    while (cp > InputBuf) {
+	if ( ((*cp & CHAR) == ' ') && ((cp[-1] & CHAR) != '\\') )
+	    break;
+	cp--;
+    }
+
+    for (p = cp, kp = KillBuf; p < Cursor; p++)	{/* save the text */
+	if ( ( *p & CHAR ) == '/') {
+	    *kp++ = '\\';
+	    *kp++ = '\\';
+	}
+	else
+	    *kp++ = *p;
+    }
+    LastKill = kp;
+
+    c_delbefore((int)(Cursor - cp));	/* delete before dot */
+    Cursor = cp;
+    if (Cursor < InputBuf)
+	Cursor = InputBuf;	/* bounds check */
+    return(e_yank_kill(c));
+}
+extern void ConsolePageUpOrDown(BOOL);
+CCRETVAL
+e_page_up(c) //blukas@broadcom.com
+    int c;
+{
+    USE(c);
+	ConsolePageUpOrDown(TRUE);
+    return (CC_ERROR);
+}
+CCRETVAL
+e_page_down(c)
+    int c;
+{
+    USE(c);
+	ConsolePageUpOrDown(FALSE);
+    return (CC_ERROR);
+}
