@@ -67,12 +67,12 @@ dprintf(char *format, ...)
  */
 int fork_copy_user_mem(HANDLE hproc) {
 	
-	int bytes,rc;
-	int size;
+	DWORD bytes,rc;
+	size_t size;
 
 	size =(char*)&bookend2 - (char*)&bookend1;
 	rc =WriteProcessMemory(hproc,&bookend1,&bookend1,
-					size,
+					(DWORD)size,
 					&bytes);
 
 	if (!rc) {
