@@ -1205,7 +1205,7 @@ Char nt_translate_bindkey(Char*s) {
 		if (!lstrcmpi("pgup",str)) {
 			keycode =  (NT_SPECIFIC_BINDING_OFFSET + KEYPAD_MAPPING_BEGIN);
 		}
-		else if (!lstrcmpi("pgdn",str)) {
+		else if (!lstrcmpi("pgdown",str)) {
 			keycode = (NT_SPECIFIC_BINDING_OFFSET + KEYPAD_MAPPING_BEGIN + 1);
 		}
 		else if (!lstrcmpi("end",str)) {
