@@ -49,6 +49,11 @@
 extern DWORD gdwPlatform;
 
 extern	 int StrQcmp(Char *, Char *);
+extern int hashval_extern(Char*);
+extern int bit_extern(int,int);
+extern void bis_extern(int,int);
+extern int hashname(Char*);
+
 BOOL is_url(const char *cmd);
 
 void error(char * ) ;
@@ -62,6 +67,7 @@ struct	biltins nt_bfunc[] = {
 #endif /* NTDBG */
     { "ps",	dops,	0,	1	},
     { "shutdown",	doshutdown,	0,	2	},
+    { "stacksize",	dostacksize,0,	1	},
     { "start",		dostart,	1,	INF	},
     { "title",		dotitle,	1,	INF	},
 };
@@ -106,7 +112,7 @@ void nt_print_builtins(unsigned int maxwidth) {
 	oldmax = maxwidth;
 
 	for ( b = nt_bfunc; b < &nt_bfunc[nt_nbfunc]; ++b)
-		maxwidth = max(maxwidth, strlen(b->bname));
+		maxwidth = max(maxwidth, (int)lstrlen(b->bname));
 
 	if (oldmax != maxwidth)
 		++maxwidth;					/* for space */
@@ -119,7 +125,7 @@ void nt_print_builtins(unsigned int maxwidth) {
 	for (b = nt_bfunc, row = 0; row < rows; row++) {
 		for (col = 0; col < columns; col++) {
 			if (b < &nt_bfunc[nt_nbfunc]) {
-				w = strlen(b->bname);
+				w = (int)lstrlen(b->bname);
 				xprintf("%s", b->bname);
 				if (col < (columns - 1))	/* Not last column? */
 					for (; w < maxwidth; w++)
@@ -133,6 +139,12 @@ void nt_print_builtins(unsigned int maxwidth) {
 	}
 
 }
+/* patch from TAGA Nayuta for start . */
+BOOL is_directory(const char *cmd) {
+	DWORD attr = GetFileAttributes(cmd);
+	return (attr != 0xFFFFFFFF &&
+			(attr & FILE_ATTRIBUTE_DIRECTORY));
+}
 void dostart(Char ** vc, struct command *c) {
 
 	char *cmdstr,*cmdend,*ptr;
@@ -229,21 +241,15 @@ void dostart(Char ** vc, struct command *c) {
 	*cmdend = 0;
 	lstrcpyn(argv0,v[k],255);
 
-	{
-		DWORD len;
 
-		/* 
-		 * strictly speaking, it should do no harm to set the path
-		 * back to '\'-delimited even in the parent, but in the
-		 * interest of consistency, we save the old value and restore it
-		 * later
-		 */
-		len = GetEnvironmentVariable("PATH",NULL,0);
-		savepath = heap_alloc(len+1);
-		GetEnvironmentVariable("PATH",savepath,len);
+	/* 
+	 * strictly speaking, it should do no harm to set the path
+	 * back to '\'-delimited even in the parent, but in the
+	 * interest of consistency, we save the old value and restore it
+	 * later
+	 */
 
-		fix_path_for_child();
-	}
+	savepath = fix_path_for_child();
 
 	if (! CreateProcess(NULL,
 						cmdstr,
@@ -256,12 +262,14 @@ void dostart(Char ** vc, struct command *c) {
 						&si,
 						&pi) ) {
 
-		SetEnvironmentVariable("PATH",savepath);
-		heap_free(savepath);
+		restore_path(savepath);
 
 		ret = GetLastError();
 		if (ret == ERROR_BAD_EXE_FORMAT || ret == ERROR_ACCESS_DENIED ||
-				(ret == ERROR_FILE_NOT_FOUND && is_url(v[k])) ) {
+				(ret == ERROR_FILE_NOT_FOUND && 
+					(is_url(v[k]) || is_directory(v[k]))
+				) 
+			) {
 
 			errno = ENOEXEC;
 
@@ -302,16 +310,15 @@ void dostart(Char ** vc, struct command *c) {
 		CloseHandle(pi.hProcess);
 		CloseHandle(pi.hThread);
 
-		SetEnvironmentVariable("PATH",savepath);
 		heap_free(cmdstr);
-		heap_free(savepath);
+		restore_path(savepath);
 	}
 	blkfree((Char **)v);
 	return;
 }
 void error(char * ebuf) {
 
-	write(2,ebuf,strlen(ebuf));
+	write(2,ebuf,(int)lstrlen(ebuf));
 }
 void make_err_str(unsigned int error,char *buf,int size) {
 
@@ -341,10 +348,6 @@ void nt_set_env(Char *name, Char *val) {
 	cname = name?short2str(name):NULL;
 	if(cname) {
 		len = lstrlen(cname);
-		/*
-		if (len >= BUFSIZ) 
-			stderror(ERR_TOOLARGE);
-		*/
 		nameBuf = heap_alloc(len+1);
 		if (!nameBuf) {
 			stderror(ERR_TOOLARGE);
@@ -355,15 +358,14 @@ void nt_set_env(Char *name, Char *val) {
 	if(cval) {
 		len = lstrlen(cval);
 		valBuf = heap_alloc(len+1);
-		/*
-		if (len >= BUFSIZ) 
-			stderror(ERR_TOOLARGE);
-		*/
 		lstrcpy(valBuf,cval);
 	}
 	
 	SetEnvironmentVariable(nameBuf,cval?valBuf:NULL);
 
+	if (!lstrcmp(nameBuf,"TCSHONLYSTARTEXES")) 
+		init_shell_dll();
+
 	heap_free(nameBuf);
 	if (cval)
 		heap_free(valBuf);
@@ -478,6 +480,9 @@ int nt_feed_to_cmd(char *file,char **argv) {
 		lstrcat(ptr,*argv);
 		argv++;
 	}
+
+	ptr = fix_path_for_child();
+
 	if (!CreateProcess(NULL,
 						cmdbuf,
 						NULL,
@@ -488,8 +493,11 @@ int nt_feed_to_cmd(char *file,char **argv) {
 						NULL,
 						&si,
 						&pi) ){
+		restore_path(ptr);
 		return 1;
 	}
+
+	restore_path(ptr);
 	CloseHandle(pi.hThread);
 	WaitForSingleObject(pi.hProcess,INFINITE);
 	CloseHandle(pi.hProcess);
@@ -499,7 +507,8 @@ int nt_feed_to_cmd(char *file,char **argv) {
 }
 static char *hb_subst_array[20] ;
 void init_hb_subst(void) {
-	int i= 0,len;
+	int i= 0;
+	size_t len;
 	char envbuf[1024];
 	char *ptr;
 	char *p2;
@@ -523,7 +532,7 @@ void init_hb_subst(void) {
 				continue;
 			}
 			hb_subst_array[i] = heap_alloc(len+1);
-			strncpy(hb_subst_array[i],p2,len);
+			strncpy(hb_subst_array[i],p2,(int)len);
 
 			i++;
 			p2 = ptr+1;
@@ -564,14 +573,17 @@ static char *no_assoc_array[20];
 
 void init_shell_dll(void) {
 
-	int rc,i,len;
+	int rc,i;
+	size_t len;
 	char *p2, *ptr;
 
-	hShellDll = LoadLibrary("Shell32.dll");
-	if (hShellDll) {
-		pShellExecuteEx = (shell_ex_func)GetProcAddress(
-									hShellDll,
-									"ShellExecuteEx");
+	if (!hShellDll) {
+		hShellDll = LoadLibrary("Shell32.dll");
+		if (hShellDll) {
+			pShellExecuteEx = (shell_ex_func)GetProcAddress(
+					hShellDll,
+					"ShellExecuteEx");
+		}
 	}
 	rc=GetEnvironmentVariable("TCSHONLYSTARTEXES",no_assoc,256) ;
 	if (!rc || (rc > 255))
@@ -611,10 +623,6 @@ void init_shell_dll(void) {
 		dprintf("no_assoc array %d inited remains %s\n",i,no_assoc_array[i]);
 #endif NTDBG
 
-
-
-	//if (getenv("TCSHONLYSTARTEXES") != NULL)
-	//	__nt_only_start_exes = 1;
 }
 // return non-zero if str is found in no_assoc_array
 int find_no_assoc(char *str) {
@@ -695,13 +703,23 @@ void try_shell_ex(char **argv,int exitsuccess) {
 	shinfo.nShow = SW_SHOWDEFAULT;
 
 
-	if ( (rc =pShellExecuteEx(&shinfo)) && exitsuccess)
+	ptr = fix_path_for_child();
+
+	if ( (rc =pShellExecuteEx(&shinfo)) && exitsuccess) {
 		ExitProcess(0);
+	}
 	else if (!rc && !exitsuccess) {
+
 		make_err_str(GetLastError(),cmdstr,512);
 		wsprintf(err2,"%s",prog);
+
+		restore_path(ptr);
+
 		stderror(ERR_SYSTEM,err2,cmdstr);
 	}
+
+	restore_path(ptr);
+
 	if (!rc && exitsuccess)
 		errno = ENOEXEC;
 	else
@@ -714,8 +732,6 @@ void dodebugbreak(Char **vc, struct command *c) {
 }
 #endif NTDBG
 int nt_texec(char *, char**) ;
-extern int hashval_extern(Char*);
-extern int bit_extern(int,int);
 static Char *epath;
 static Char *abspath[] = {STRNULL,0};
 int nt_try_fast_exec(struct command *t) {
@@ -836,7 +852,7 @@ int nt_texec(char *prog, char**args ) {
     DWORD type=0;
 	DWORD dwCreationflags;
 	unsigned int priority;
-	char *argv0;
+	char *argv0, *savepath;
 	char *cmdstr,*cmdend ,*cmdstr_save;
 	unsigned int cmdsize,cmdlen;
     char *p2;
@@ -922,6 +938,9 @@ int nt_texec(char *prog, char**args ) {
 	}
 re_cp:
 	dprintf("cmdstr %s\n",cmdstr);
+
+	savepath = fix_path_for_child();
+
 	if (!CreateProcess(argv0,
 					   cmdstr,
 					   NULL,
@@ -951,6 +970,7 @@ re_cp:
 	}
 	else{
 		int gui_app ;
+		DWORD exitcode;
 		char guivar[50];
 
 
@@ -959,11 +979,15 @@ re_cp:
 		else
 			gui_app= is_gui(argv0);
 		
-		if(!gui_app)
+		if(!gui_app) {
 			WaitForSingleObject(pi.hProcess,INFINITE);
+			(void)GetExitCodeProcess(pi.hProcess,&exitcode);
+			set(STRstatus, putn(exitcode), VAR_READWRITE);
+		}
 		retval = 0;
 	}
 cleanup:
+	restore_path(savepath);
 	heap_free(cmdstr_save);
 	if (argv0)
 		heap_free(argv0);
@@ -985,7 +1009,7 @@ BOOL is_url(const char *cmd) {
   DWORD size;
 
   c = strchr(cmd, ':');
-  size = c - cmd;
+  size = (DWORD)(c - cmd);
   if (!c || size <= 1)
 	return FALSE;
 
@@ -1011,3 +1035,138 @@ BOOL is_url(const char *cmd) {
   RegCloseKey(hkey);
   return TRUE;
 }
+void dostacksize(Char ** vc, struct command *c) {
+	int k;
+	char **v;
+	USE(c);
+	*vc++;
+	gflag = 0;
+	tglob(vc);
+	if (gflag) {
+		vc = globall(vc);
+		if (vc == 0)
+			stderror(ERR_NAME | ERR_NOMATCH);
+	}
+	else
+		vc = gargv = saveblk(vc);
+	trim(vc);
+
+	v = short2blk(vc);
+	if (v[0] == NULL) {
+		xprintf("Stack limit for shell threads is %d bytes\n",gdwStackSize);
+	}
+	else {
+		for (k = 0; v[k] != NULL ; k++){
+			gdwStackSize = (unsigned)atol(v[k]);
+		}
+	}
+	blkfree((Char **)v);
+}
+/*
+ * patch based on work by Chun-Pong Yu (bol.pacific.net.sg)
+ */
+BOOL is_nt_executable(char *path,char *extension) {
+	DWORD exetype;
+
+	if (GetBinaryType(path,&exetype))
+		return TRUE;
+	if (*extension && find_no_assoc(extension))
+		return TRUE;
+
+	return FALSE;
+}
+int
+executable(dir, name, dir_ok)
+    Char   *dir, *name;
+    bool    dir_ok;
+{
+	struct stat stbuf;
+	Char    path[MAXPATHLEN + 1];
+	char   *strname;
+	char extension[MAXPATHLEN];
+	char *ptr = short2str(path);
+	char *p2 = ptr;
+	int has_ext = 0;
+	extern void copyn(Char *, Char *, int);
+	extern void catn(Char *, Char *, int);
+
+	(void) memset(path, 0, sizeof(path));
+
+	if (dir && *dir) {
+		copyn(path, dir, MAXPATHLEN);
+		catn(path, name, MAXPATHLEN);
+
+		while (*ptr++)
+			continue;
+		--ptr;
+
+		while(ptr > p2) { 
+			if (*ptr == '/')
+				break;
+			if (*ptr == '.') {
+				has_ext = 1;
+				lstrcpyn(extension,ptr+1,MAXPATHLEN);
+				break;
+			}
+			ptr--;
+		}
+		if (!has_ext && (nt_stat(p2, &stbuf) == -1))
+			catn(path, STRdotEXE, MAXPATHLEN);
+		strname = short2str(path);
+	}
+	else
+		strname = short2str(name);
+
+	return (stat(strname, &stbuf) != -1 &&
+			((dir_ok && S_ISDIR(stbuf.st_mode)) ||
+			 (S_ISREG(stbuf.st_mode) &&
+			  (is_nt_executable(strname,extension) ||
+			   (stbuf.st_mode & (S_IXOTH | S_IXGRP | S_IXUSR)))
+			 )));
+}
+int
+nt_check_if_windir(path)
+    char *path;
+{
+    char windir[MAX_PATH];
+
+    (void)GetWindowsDirectory(windir, sizeof(windir));
+    windir[2] = '/';
+
+    return (strstr(path, windir) != NULL);
+}
+
+void
+nt_check_name_and_hash(is_windir, file, i)
+    int is_windir;
+    char *file;
+    int i;
+{
+	char name_only[MAX_PATH];
+	char *tmp = (char *)strrchr(file, '.');
+	char uptmp[5], *nameptr, *np2;
+	int icount, hashval;
+
+	if(!tmp || tmp[4]) 
+		goto nodot;
+
+	for (icount = 0; icount < 4; icount++)
+		uptmp[icount] = toupper(tmp[icount]);
+	uptmp[4]=0;
+
+	if (is_windir)
+		if((uptmp[1] != 'E') || (uptmp[2] != 'X') || (uptmp[3] != 'E'))
+			return;
+	(void) memset(name_only, 0, MAX_PATH);
+	nameptr = file;
+	np2 = name_only;
+	while(nameptr != tmp) {
+		*np2++= tolower(*nameptr);
+		nameptr++;
+	}
+	hashval = hashname(str2short(name_only));
+	bis_extern(hashval, i);
+nodot:
+	hashval = hashname(str2short(file));
+	bis_extern(hashval, i);
+}
