@@ -220,7 +220,7 @@ typedef int mode_t;
 typedef void sigret_t;
 typedef int speed_t;
 typedef unsigned char u_char;
-typedef unsigned long caddr_t;
+typedef size_t caddr_t;
 struct timeval{
 	long tv_sec;
 	long tv_usec;
@@ -264,6 +264,14 @@ struct group {
 	  char    **gr_mem;        /* group members */
 };
 
+#ifndef _INTPTR_T_DEFINED
+#ifdef  _WIN64
+typedef __int64             intptr_t;
+#else
+typedef int                 intptr_t;
+#endif
+#define _INTPTR_T_DEFINED
+#endif
 /* ntport.c */
 extern char *			ttyname(int);
 extern struct passwd*  getpwuid(uid_t ) ;
@@ -293,11 +301,14 @@ extern int  nt_chdir(char*);
 extern void  nt_execve(char *,char**,char**);
 extern void  nt_exec(char *,char**);
 extern void quoteProtect(char *, char *) ;
-extern void fix_path_for_child(void) ;
+extern char* fix_path_for_child(void) ;
+extern void restore_path(char *) ;
 extern int copy_quote_and_fix_slashes(char *,char *, int * );
 extern void concat_args_and_quote(char **,char **,unsigned int *, char **, 
 	unsigned int *) ;
 
+
+extern int is_nt_executable(char*,char*);
 /* io.c */
 extern int  force_read(int, unsigned char*,int);
 extern int  nt_read(int, unsigned char*,int);
@@ -328,8 +339,8 @@ extern int  nt_puts(char*);
 extern int  nt_putchar(char);
 extern int  nt_fclose(FILE*);
 extern int  nt_fputs(char *, FILE*);
-extern unsigned long  __nt_get_osfhandle(int);
-extern int __nt_open_osfhandle(long, int);
+extern intptr_t  __nt_get_osfhandle(int);
+extern int __nt_open_osfhandle(intptr_t, int);
 extern int nt_clearerr(FILE*);
 extern int  nt_dup2(int,int );
 extern FILE* nt_fdopen(int,char*);
@@ -352,8 +363,6 @@ extern void init_stdio(void) ;
 extern void *my_stdin,*my_stdout,*my_stderr;
 #endif STDIO_C
 
-//int   gettimeofday(struct timeval *, struct timezone *) ;
-//char * getlogin(void);
 
 /* nt.char.c */
 extern unsigned char oem_it(unsigned char );
@@ -412,7 +421,10 @@ extern void cleanup_netbios(void);
 extern void dostart(short **,struct command *);
 extern void docls(short **,struct command *);
 extern void dotitle(short **, struct command * ) ;
+extern void dostacksize(short**,struct command *);
+#ifdef NTDBG
 extern void dodebugbreak(short **, struct command * ) ;
+#endif NTDBG
 extern void nt_set_env(short *, short*);
 extern char *hb_subst(char *) ;
 extern void init_hb_subst() ;
@@ -444,6 +456,7 @@ extern unsigned short __nt_child_nohupped;
 extern DWORD gdwPlatform,gdwVersion;
 extern int is_dev_clipboard_active;
 extern HANDLE ghdevclipthread;
+extern DWORD gdwStackSize;
 
 // bogus
 #define getppid() 0
