@@ -43,8 +43,6 @@
 #include <sh.h>
 #include <errno.h>
 
-#pragma warning(disable:4028)
-#pragma warning(disable:4024)
 
 #define REGKEY_PERF         "software\\microsoft\\windows nt\\currentversion\\perflib"
 #define REGSUBKEY_COUNTERS  "Counters"
@@ -61,7 +59,6 @@ typedef struct _pslist {
 
 typedef BOOL (WINAPI *walker)(HANDLE,LPPROCESSENTRY32);
 typedef HANDLE (WINAPI *create_snapshot)(DWORD,DWORD);
-typedef DWORD (*plist_proc)(void);
 static walker proc32First;
 static walker proc32Next;
 static create_snapshot createSnapshot;
@@ -75,6 +72,7 @@ static enummod enum_process_modules;
 static getfilename_ex getfilenameex;
 static getbasename GetModuleBaseNameA;
 
+typedef DWORD (*plist_proc)(void);
 
 DWORD Win95Lister(void);
 DWORD NTLister(void);
@@ -88,7 +86,7 @@ static HMODULE hlib;
 extern DWORD gdwPlatform;
 extern void make_err_str(int,char *,int);
 
-BOOL CALLBACK enum_wincb(HWND hwnd,DWORD nump) {
+BOOL CALLBACK enum_wincb(HWND hwnd,LPARAM nump) {
 
 	unsigned int i;
 	DWORD pid = 0;
@@ -109,7 +107,7 @@ BOOL CALLBACK enum_wincb(HWND hwnd,DWORD nump) {
 	return TRUE;
 }
 static HWND ghwndtokillbywm_close;
-BOOL CALLBACK enum_wincb2(HWND hwnd,DWORD pidtokill) {
+BOOL CALLBACK enum_wincb2(HWND hwnd,LPARAM pidtokill) {
 	DWORD pid = 0;
 
 	if (!GetWindowThreadProcessId(hwnd,&pid))
@@ -123,7 +121,7 @@ BOOL CALLBACK enum_wincb2(HWND hwnd,DWORD pidtokill) {
 	return TRUE;
 }
 int kill_by_wm_close(int pid)  {
-	EnumWindows(enum_wincb2,pid);
+	EnumWindows(enum_wincb2,(LPARAM)pid);
 	if (!ghwndtokillbywm_close)
 		return -1;
 	ghwndtokillbywm_close = NULL;
@@ -131,7 +129,6 @@ int kill_by_wm_close(int pid)  {
 }
 DWORD Win95Lister(void) {
 
-
 	HANDLE hsnap;
 	PROCESSENTRY32 pe;
 	unsigned long nump =0;
@@ -163,14 +160,13 @@ done:
 	CloseHandle(hsnap);
 
 	if (g_dowindows) {
-		EnumWindows(enum_wincb,nump);
+		EnumWindows(enum_wincb,(LPARAM)nump);
 	}
 	return nump;
 }
 
 DWORD NTLister(void) {
 	
-
 	DWORD procs[200],dummy,ignore;
 	HANDLE hproc;
 	HMODULE hmod;
@@ -209,7 +205,7 @@ DWORD NTLister(void) {
 		
 	}
 	if (g_dowindows) {
-		EnumWindows(enum_wincb,dummy);
+		EnumWindows(enum_wincb,(LPARAM)dummy);
 	}
 	return dummy;
 }
@@ -387,7 +383,7 @@ DWORD NTLister_old_style( void) {
 	}
 
 	if (g_dowindows) {
-		EnumWindows(enum_wincb,numprocesses);
+		EnumWindows(enum_wincb,(LPARAM)numprocesses);
 	}
 exit:
 	if (buf) {
@@ -430,6 +426,7 @@ void init_plister(void) {
 		if (!hlib)
 			return ;
 
+
 		ProcessListFunc = Win95Lister;
 
 		proc32First = (walker)GetProcAddress(hlib,"Process32First");
@@ -442,6 +439,8 @@ void init_plister(void) {
 			ProcessListFunc = NULL;
 		}
 	}
+	if (hlib)
+		FreeLibrary(hlib);
 
 }
 void dops(Char ** vc, struct command *c) {
@@ -506,7 +505,7 @@ void doshutdown(Char **vc, struct command *c) {
 	DWORD cbPrevious = sizeof(TOKEN_PRIVILEGES);
 
 	if (gdwPlatform != VER_PLATFORM_WIN32_NT) {
-		stderror(ERR_SYSTEM,"Shutdown","Sorry,not supported on win95");
+		stderror(ERR_SYSTEM,"shutdown","Sorry,not supported on win95");
 	}
 
 	shutdown_ok = reboot = shutdown = logoff = 0;
