@@ -244,7 +244,8 @@ DWORD WINAPI rsp(void) {
 void restore_main_ctx(void) {
 	HANDLE ht;
 	DWORD tid;
-	ht = CreateThread(NULL,0,(LPTHREAD_START_ROUTINE)rsp,NULL,0,&tid);
+	ht = CreateThread(NULL,gdwStackSize,
+				(LPTHREAD_START_ROUTINE)rsp,NULL,0,&tid);
 	CloseHandle(ht);
 	while(1) {
 		Sleep(10);
@@ -488,8 +489,8 @@ unsigned int alarm(unsigned int seconds) {
 	}
 	__alarm_set = 1;
 
-	ht = CreateThread(NULL,0, (LPTHREAD_START_ROUTINE)alarm_callback,
-					(void*)seconds,0,&tid);
+	ht = CreateThread(NULL,gdwStackSize,
+				(LPTHREAD_START_ROUTINE)alarm_callback, (void*)seconds,0,&tid);
 	if (ht)
 		CloseHandle(ht);
 	
@@ -560,7 +561,7 @@ void start_sigchild_thread(HANDLE hproc, DWORD pid) {
 	args->pid = pid;
 
 	hthr = CreateThread(NULL,
-							0,
+							gdwStackSize,
 							(LPTHREAD_START_ROUTINE)sigchild_thread,
 							(LPVOID)args,
 							0,
