@@ -56,13 +56,13 @@
 #define FCONSOLE 2
 
 typedef struct _myfile {
-	unsigned long  handle;
+	HANDLE  handle;
 	unsigned long flags;
 } MY_FILE;
 
 typedef unsigned long u_long;
 
-#define INVHL (DWORD)(INVALID_HANDLE_VALUE)
+#define INVHL (INVALID_HANDLE_VALUE)
 
 MY_FILE __gOpenFiles[__MAX_OPEN_FILES]={0};
 MY_FILE __gOpenFilesCopy[__MAX_OPEN_FILES]={0};
@@ -75,9 +75,9 @@ int __dup_stdin = 0;
 void init_stdio(void) {
 
 	int i;
-	__gOpenFiles[0].handle = (u_long)GetStdHandle(STD_INPUT_HANDLE);
-	__gOpenFiles[1].handle = (u_long)GetStdHandle(STD_OUTPUT_HANDLE);
-	__gOpenFiles[2].handle = (u_long)GetStdHandle(STD_ERROR_HANDLE);
+	__gOpenFiles[0].handle = GetStdHandle(STD_INPUT_HANDLE);
+	__gOpenFiles[1].handle = GetStdHandle(STD_OUTPUT_HANDLE);
+	__gOpenFiles[2].handle = GetStdHandle(STD_ERROR_HANDLE);
 
 	__gOpenFiles[0].flags = (GetFileType((HANDLE)STD_INPUT_HANDLE)== 
 									FILE_TYPE_CHAR)?  FCONSOLE:0;
@@ -145,8 +145,10 @@ void copy_fds(void ) {
 	if (didfds || __dup_stdin)
 		min =0;
 	for(i=min;i<__MAX_OPEN_FILES;i++) {
-		if (__gOpenFiles[i].handle == INVHL)
+		if (__gOpenFiles[i].handle == INVHL) {
+			__gOpenFilesCopy[i].handle = INVHL;
 			continue;
+		}
 		
 		if(!DuplicateHandle(GetCurrentProcess(), 
 						(HANDLE)__gOpenFiles[i].handle ,
@@ -157,15 +159,15 @@ void copy_fds(void ) {
 		__gOpenFilesCopy[i].flags = __gOpenFiles[i].flags;
 	}
 }
-u_long __nt_get_osfhandle(int fd) {
-	return (u_long)(__gOpenFiles[fd].handle);
+intptr_t __nt_get_osfhandle(int fd) {
+	return (intptr_t)(__gOpenFiles[fd].handle);
 }
-int __nt_open_osfhandle(long h1, int mode) {
+int __nt_open_osfhandle(intptr_t h1, int mode) {
 	int i;
 
 	for(i=0;i<__MAX_OPEN_FILES;i++) {
 		if (__gOpenFiles[i].handle == INVHL) {
-			__gOpenFiles[i].handle = h1;
+			__gOpenFiles[i].handle = (HANDLE)h1;
 			__gOpenFiles[i].flags = 0;
 			return i;
 		}
@@ -269,32 +271,33 @@ int nt_seek(HANDLE h1, long offset, int how) {
 	return 0;
 }
 int nt_lseek(int fd,long offset, int how) {
-	u_long h1 ; 
+	HANDLE h1 ; 
 	h1 =__gOpenFiles[fd].handle;
 	return nt_seek((HANDLE)h1,offset,how);
 }
 int nt_isatty(int fd) {
-	u_long h1 = __gOpenFiles[fd].handle;
+	HANDLE h1 = __gOpenFiles[fd].handle;
 	return (__gOpenFiles[fd].flags & FCONSOLE);
 }
 int nt_dup(int fdin) {
 
 	HANDLE hdup;
-	u_long horig =  __gOpenFiles[fdin].handle;
+	HANDLE horig =  __gOpenFiles[fdin].handle;
 	int ret;
 	
 	if (!DuplicateHandle(GetCurrentProcess(),
-						 (HANDLE)horig,
+						 horig,
 						 GetCurrentProcess(),
 						 &hdup,
 						 0,
 						 FALSE,
 						 DUPLICATE_SAME_ACCESS)) {
+		errno = GetLastError();
 		errno = EBADF;
 		return -1;
 	}
 //	dprintf("orig 0x%08x duped to 0x%08x\n",horig,hdup);
-	ret = __nt_open_osfhandle((long)hdup,_O_BINARY | _O_NOINHERIT);
+	ret = __nt_open_osfhandle((intptr_t)hdup,_O_BINARY | _O_NOINHERIT);
 
 	__gOpenFiles[ret].flags = __gOpenFiles[fdin].flags;
 
@@ -303,7 +306,7 @@ int nt_dup(int fdin) {
 int nt_dup2(int fdorig,int fdcopy) {
 
 	HANDLE hdup;
-	u_long horig =  __gOpenFiles[fdorig].handle;
+	HANDLE horig =  __gOpenFiles[fdorig].handle;
 	
 	if (__gOpenFiles[fdcopy].handle != INVHL) {
 		CloseHandle((HANDLE)__gOpenFiles[fdcopy].handle );
@@ -311,16 +314,17 @@ int nt_dup2(int fdorig,int fdcopy) {
 		__gOpenFiles[fdcopy].flags = 0;
 	}
 	if (!DuplicateHandle(GetCurrentProcess(),
-						 (HANDLE)horig,
+						 horig,
 						 GetCurrentProcess(),
 						 &hdup,
 						 0,
 						 fdcopy<3?TRUE:FALSE,
 						 DUPLICATE_SAME_ACCESS)) {
+		errno = GetLastError();
 		errno = EBADF;
 		return -1;
 	}
-	__gOpenFiles[fdcopy].handle = (u_long)hdup;
+	__gOpenFiles[fdcopy].handle = hdup;
 	__gOpenFiles[fdcopy].flags = __gOpenFiles[fdorig].flags;
 	switch(fdcopy) {
 		case 0:
@@ -352,8 +356,8 @@ int nt_pipe(int hpipe[2]) {
 	HANDLE hpipe2[2];
 
 	nt_pipe2(hpipe2);
-	hpipe[0] = _open_osfhandle((long)hpipe2[0],O_NOINHERIT);
-	hpipe[1] = _open_osfhandle((long)hpipe2[1],O_NOINHERIT);
+	hpipe[0] = _open_osfhandle((intptr_t)hpipe2[0],O_NOINHERIT);
+	hpipe[1] = _open_osfhandle((intptr_t)hpipe2[1],O_NOINHERIT);
 	return 0;
 }
 /* check if name is //server. if checkifShare is set,
@@ -374,7 +378,12 @@ int is_server(char *name,int checkifShare) {
 	
 	p2 = p1 + 2;
 	while (*p2 && *p2 != '/' && *p2 != '\\')
-		p2++;
+#ifdef DSPMBYTE
+		if (Ismbyte1(*p2) && *(p2 + 1))
+			p2 += 2;
+		else
+#endif /* DSPMBYTE */
+			p2++;
 		
 	/* just check for server */
 	if (!checkifShare) {
@@ -481,7 +490,7 @@ int nt_creat(char *filename, int mode) {
 		return -1;
 	}
 get_fd:
-	fd = _open_osfhandle((long)retval,_O_BINARY);
+	fd = _open_osfhandle((intptr_t)retval,_O_BINARY);
 	if (fd <0) {
 		//should never happen
 		abort();
@@ -578,7 +587,7 @@ int nt_open(char *filename, int perms,...) {
 		SetFilePointer(retval,0,NULL,FILE_END);
 	}
 get_fd:
-	fd = _open_osfhandle((long)retval,_O_BINARY);
+	fd = _open_osfhandle((intptr_t)retval,_O_BINARY);
 	if (fd <0) {
 		//should never happen
 		abort();
@@ -610,7 +619,7 @@ int nt_fstat(int fd, struct stat *stbuf) {
 						FALSE,
 						DUPLICATE_SAME_ACCESS) )
 		return -1;
-	realfd = _open_osfhandle((long)h1,0);
+	realfd = _open_osfhandle((intptr_t)h1,0);
 	if (realfd <0 ) 
 		return -1;
 	
