@@ -52,10 +52,20 @@
 
 DWORD gdwPlatform,gdwVersion;
 unsigned short __nt_really_exec = 0,__nt_child_nohupped =0;
+DWORD gdwStackSize = 524288;//0.5 MB
 
-
-
-
+void path_slashify(char *pstr) {
+	while(*pstr) {
+#ifdef DSPMBYTE
+		if (Ismbyte1(*pstr) && *(pstr + 1))
+			pstr ++;
+		else
+#endif /* DSPMBYTE */
+		if (*pstr == '\\') 
+			*pstr = '/';
+		pstr++;
+	}
+}
 void nt_init(void) {
 
 
@@ -85,10 +95,51 @@ void nt_cleanup(void){
 	nt_cleanup_signals();
 	cleanup_netbios();
 }
+void caseify_pwd(char *curwd) {
+	char *sp, *dp, p,*s;
+	WIN32_FIND_DATA fdata;
+	HANDLE hFind;
+
+	if (gdwPlatform !=VER_PLATFORM_WIN32_NT) 
+		return;
+	
+	if (*curwd == '\\' && (!curwd[1] || curwd[1] == '\\'))
+		return;
+	sp = curwd +3;
+	dp = curwd +3;
+	do {
+		p= *sp;
+		if (p && p != '\\'){
+			sp++;
+			continue;
+		}
+		else {
+			*sp = 0;
+			hFind = FindFirstFile(curwd,&fdata);
+			*sp = p;
+			if (hFind != INVALID_HANDLE_VALUE) {
+				FindClose(hFind);
+				s = fdata.cFileName;	
+				while(*s) {
+					*dp++ = *s++;
+				}
+				dp++;
+				sp = dp;
+			}
+			else {
+				sp++;
+				dp = sp;
+			}
+		}
+		sp++;
+	}while(p != 0);
+
+}
 static unsigned char defcwd[MAX_PATH];
 char * forward_slash_get_cwd(char * path, int maxlen) {
 
 	char *ptemp;
+	Char *vp;
 	int rc ;
 	
 	if ((path == NULL) || (maxlen == 0)) {
@@ -101,11 +152,26 @@ char * forward_slash_get_cwd(char * path, int maxlen) {
 		errno = ERANGE;
 		return NULL;
 	}
+	vp = varval(STRNTcaseifypwd);
+	if (vp != STRNULL) {
+		caseify_pwd(path);
+	}
 	ptemp=path;
+
+	path_slashify(ptemp);
+
+#if 0
 	while(*ptemp) {
-		if (*ptemp == '\\') *ptemp = '/';
+#ifdef DSPMBYTE
+		if (Ismbyte1(*ptemp) && *(ptemp + 1))
+			ptemp ++;
+		else
+#endif /* DSPMBYTE */
+		if (*ptemp == '\\') 
+			*ptemp = '/';
 		*ptemp++;
 	}
+#endif 0
 	return path;
 }
 void getmachine (void) {
@@ -178,12 +244,7 @@ void getmachine (void) {
 
 }
 void nt_exec(char *prog, char**args) {
-//	__try {
-		nt_execve(prog,args,NULL);
-//	}__except(1) {
-//		stderror(ERR_TOOMANY);
-//	}
-
+	nt_execve(prog,args,NULL);
 }
 void nt_execve(char *prog, char**args, char**envir ) {
 
@@ -310,7 +371,7 @@ win95_directly_here:
 	}
 	priority = GetThreadPriority(GetCurrentThread());
 
-	fix_path_for_child();
+	(void)fix_path_for_child();
 
 	if (is_winnt)
 		dwCreationflags |= CREATE_SUSPENDED;
@@ -443,7 +504,6 @@ extern void heap_init(void);
 void silly_entry(void *peb) {
 	char * path1=NULL;
 	int rc;
-	char *ptr;
 	char temp[MAX_PATH+5];
 	char buf[MAX_PATH];
 	char ptr1[MAX_PATH];
@@ -467,11 +527,20 @@ void silly_entry(void *peb) {
 	/* If home is set, we only need to change '\' to '/' */
 	rc = GetEnvironmentVariable("HOME",buf,MAX_PATH);
 	if (rc && (rc < MAX_PATH)){
+		path_slashify(buf);
+#if 0
 		char *s = &buf[0];
 		while(*s){
-			if (*s == '\\') *s = '/';
+#ifdef DSPMBYTE
+			if (Ismbyte1(*s) && *(s + 1)) 
+				s++;
+			else
+#endif /* DSPMBYTE */
+			if (*s == '\\')
+				*s = '/';
 			s++;
 		}
+#endif 0
 		(void)SetEnvironmentVariable("HOME",buf);
 		goto skippy;
 	}
@@ -506,11 +575,19 @@ void silly_entry(void *peb) {
 	else {
 		MessageBox(NULL,"Unknown platform","tcsh",MB_ICONHAND);
 	}
+	path_slashify(temp);
+#if 0
 	ptr = &temp[0];
 	while(*ptr) {
-		if(*ptr == '\\') *ptr ='/';
+#ifdef DSPMBYTE
+		if (Ismbyte1(*ptr) && *(ptr + 1)) ptr ++;
+		else
+#endif /* DSPMBYTE */
+		if(*ptr == '\\') 
+			*ptr ='/';
 		ptr++;
 	}
+#endif 0
 	SetEnvironmentVariable("HOME",temp);
 
 skippy:
@@ -589,8 +666,10 @@ void concat_args_and_quote(char **args, char **cstr, unsigned int *clen,
 	char *tempptr, *cmdend ,*cmdstr;
 	short quotespace = 0;
 	short quotequote = 0;
+	short noquoteprotect = 0;
 	char tempquotedbuf[256];
 
+	noquoteprotect = (varval(STRNTnoquoteprotect) != STRNULL);
 	/* 
 		quotespace hack needed since execv() would have separated args, but
 		createproces doesnt
@@ -611,6 +690,7 @@ void concat_args_and_quote(char **args, char **cstr, unsigned int *clen,
 		arglen = 0;
 		argcount++;
 
+		dprintf("args is %s\n",*args);
 		if (!*tempptr) {
 			*cmdend++ = '"';
 			*cmdend++ = '"';
@@ -635,7 +715,7 @@ void concat_args_and_quote(char **args, char **cstr, unsigned int *clen,
 		if (quotespace)
 			*cmdend++ = '"';
 
-        if (quotequote){
+        if ((noquoteprotect == 0) && quotequote){
             tempquotedbuf[0]=0;
 
             tempptr = &tempquotedbuf[0];
@@ -670,25 +750,44 @@ void concat_args_and_quote(char **args, char **cstr, unsigned int *clen,
 
 
 }
-void fix_path_for_child(void) {
+char *fix_path_for_child(void) {
 
 	char *ptr;
 	Char *vp;
-	char pathstr[1024];
+	char *pathstr;
+	char *oldpath;
+	long len;
 
 	vp = varval(STRNTlamepathfix);
 
-	pathstr[1023] =0 ;
 	if (vp != STRNULL) {
-		if (GetEnvironmentVariable("PATH",pathstr,1024) <1024 ) {
-			ptr = pathstr;
-			while(*ptr) {
-				if (*ptr == '/')
-					*ptr = '\\';
-				ptr++;
-			}
-			SetEnvironmentVariable("PATH",pathstr);
+
+		len = GetEnvironmentVariable("PATH",NULL,0);
+
+		oldpath = heap_alloc(len+1);
+		pathstr = heap_alloc(len+1);
+
+		len = GetEnvironmentVariable("PATH",oldpath,len+1);
+		memcpy(pathstr,oldpath,len);
+
+		ptr = pathstr;
+		while(*ptr) {
+			if (*ptr == '/')
+				*ptr = '\\';
+			ptr++;
 		}
+		SetEnvironmentVariable("PATH",pathstr);
+		heap_free(pathstr);
+
+		return oldpath; //freed in restore_path;
 	}
+	else
+		return NULL;
 
 }
+void restore_path(char *oldpath) {
+	if (oldpath) {
+		SetEnvironmentVariable("PATH",oldpath);
+		heap_free(oldpath);
+	}
+}
